//=============================================================================
//
//	Copyright (C) 1996,1999 by Paul S. McCarthy and Eric Sunshine.
//		Written by Paul S. McCarthy and Eric Sunshine.
//			    All Rights Reserved.
//
//	This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the authors
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//
//=============================================================================
//-----------------------------------------------------------------------------
// MiscExporter.M
//
//	Object that exports the contents of an MiscTableScroll in
//	various useful formats.
//
// BUGS:
//	* Enable / disable save controls based on format.
// TODO:
//	* Add more export formats: rtf0, Lotus 1-2-3, Quattro, Excel, etc.
//	* Maybe provide option for <CR><LF> line terminators.
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscExporterAccessoryView.M,v 1.2 99/06/29 07:35:15 sunshine Exp $
// $Log:	MiscExporterAccessoryView.M,v $
// Revision 1.2  99/06/29  07:35:15  sunshine
// v140: Rewrote so that layout of controls is dynamic, based upon their sizes
// rather than using hard-coded coordinates.  No longer abbreviates box titles
// as "Col Titles" & "Col Grids"; now uses "Column Titles" & "Column Grid".
// 
//-----------------------------------------------------------------------------
#import "MiscExporterAccessoryView.h"

extern "Objective-C" {
#import	<appkit/Box.h>
#import	<appkit/Button.h>
#import	<appkit/Matrix.h>
#import	<appkit/Panel.h>
#import	<appkit/PopUpList.h>
}
extern "C" {
#import	<math.h>
#import	<string.h>	// strerror()
}

static char const* const MISC_FORMAT_NAME[] =
    {
    "ASCII-Fixed",
    "ASCII-Tab",
    "ASCII-Delimited",
    "dBASE III (.DBF)"
    };

static char const* const MISC_TITLE_MODE_NAME[] =
    {
    "Off",
    "On",
    "Wrap",
    "Truncate"
    };

static char const* const MISC_GRID_MODE_NAME[] =
    {
    "Off",
    "Line",
    "Space"
    };


//-----------------------------------------------------------------------------
// name_to_tag
//-----------------------------------------------------------------------------
static int name_to_tag( char const* nm, char const* const* nm_array,
			int first, int last, int fallback )
    {
    int const n = (last - first) + 1;
    for (int i = 0; i < n; i++)
	if (strcmp( nm_array[i], nm ) == 0)
	    return i + first;
    return fallback;
    }


inline static MiscExportFormat name_to_format( char const* nm )
    {
    return (MiscExportFormat)
	name_to_tag( nm, MISC_FORMAT_NAME, MISC_EXPORT_FORMAT_FIRST,
			MISC_EXPORT_FORMAT_LAST, MISC_EXPORT_FORMAT_DEFAULT );
    }


inline static MiscExportTitleMode name_to_title_mode
	( char const* nm, MiscExportTitleMode fallback )
    {
    return (MiscExportTitleMode)
	name_to_tag( nm, MISC_TITLE_MODE_NAME, MISC_EXPORT_TITLES_FIRST,
			MISC_EXPORT_TITLES_LAST, (int) fallback );
    }


inline static MiscExportTitleMode name_to_row_title_mode( char const* nm )
    { return name_to_title_mode( nm, MISC_EXPORT_TITLES_ROW_DEFAULT ); }


inline static MiscExportTitleMode name_to_col_title_mode( char const* nm )
    { return name_to_title_mode( nm, MISC_EXPORT_TITLES_COL_DEFAULT ); }



inline static MiscExportGridMode name_to_grid_mode
	( char const* nm, MiscExportGridMode fallback )
    {
    return (MiscExportGridMode)
	name_to_tag( nm, MISC_GRID_MODE_NAME, MISC_EXPORT_GRID_FIRST,
			MISC_EXPORT_GRID_LAST, (int)fallback );
    }


inline static MiscExportGridMode name_to_row_grid_mode( char const* nm )
    { return name_to_grid_mode( nm, MISC_EXPORT_GRID_ROW_DEFAULT ); }


inline static MiscExportGridMode name_to_col_grid_mode( char const* nm )
    { return name_to_grid_mode( nm, MISC_EXPORT_GRID_COL_DEFAULT ); }


//-----------------------------------------------------------------------------
// new_popup
//-----------------------------------------------------------------------------
static Button* new_popup( char const* const* names,
	int first, int last, int init_choice )
    {
    int const n = (last - first) + 1;
    PopUpList* popup = [[PopUpList alloc] init];
    for (int i = 0; i < n; i++)
	[popup addItem:names[i]];
    [[popup itemList] selectCellAt:(init_choice - first):0];
    return (Button*)NXCreatePopUpListButton( popup );
    }


//-----------------------------------------------------------------------------
// new_box
//-----------------------------------------------------------------------------
static Box* new_box( Button* button, char const* title )
    {
    NXCoord const OFFSET = 3;
    NXRect const r = {{ 0, 0 }, { 10, 10 }};
    Box* const box = [[Box alloc] initFrame:&r];
    [box setContentView:button];
    [box setOffsets:OFFSET:OFFSET];
    [box setTitle:title];
    [box sizeToFit];
    return box;
    }


//=============================================================================
// IMPLEMENTATION
//=============================================================================
@implementation MiscExporterAccessoryView
- (MiscExportFormat)format
	{ return name_to_format( [formatPop title] ); }

- (MiscExportTitleMode)rowTitleMode;
	{ return name_to_row_title_mode( [rowTitlePop title] ); }

- (MiscExportTitleMode)colTitleMode;
	{ return name_to_col_title_mode( [colTitlePop title] ); }

- (MiscExportGridMode)rowGrid;
	{ return name_to_row_grid_mode( [rowGridPop title] ); }

- (MiscExportGridMode)colGrid;
	{ return name_to_col_grid_mode( [colGridPop title] ); }


//-----------------------------------------------------------------------------
// unifyWidths::
//-----------------------------------------------------------------------------
- (NXCoord)unifyWidths:(View*)v1 :(View*)v2
    {
    NXRect r1, r2;
    [v1 getFrame:&r1];
    [v2 getFrame:&r2];
    NXCoord w = r1.size.width > r2.size.width ? r1.size.width : r2.size.width;
    [v1 sizeTo:w:r1.size.height];
    [v2 sizeTo:w:r2.size.height];
    return w;
    }


//-----------------------------------------------------------------------------
// -initFrame:format:rowTitle:colTitle:rowGrid:colGrid:
//
// Layout of boxes containing pop-up lists is as follows:
//
//	******** FORMAT ********
//	*ROW TITLES*  *ROW GRID*
//	*COL TITLES*  *COL GRID*
//
// The origin of the column titles box is at (0,BOX_SPACING) within the 
// accessory view.  BOX_SPACING provides a needed gap between the bottom of
// the column titles box and the standard buttons (Okay, Cancel) on the Save
// Panel.
//-----------------------------------------------------------------------------
- (id)initFrame:(NXRect const*)rect
    format:(MiscExportFormat)format
    rowTitle:(MiscExportTitleMode)rowTitle
    colTitle:(MiscExportTitleMode)colTitle
    rowGrid:(MiscExportGridMode)rowGrid
    colGrid:(MiscExportGridMode)colGrid
    {
    [super initFrame:rect];

    formatPop	= new_popup( MISC_FORMAT_NAME, MISC_EXPORT_FORMAT_FIRST,
			MISC_EXPORT_FORMAT_LAST, format );
    rowTitlePop	= new_popup( MISC_TITLE_MODE_NAME, MISC_EXPORT_TITLES_FIRST,
			MISC_EXPORT_TITLES_LAST, rowTitle );
    colTitlePop	= new_popup( MISC_TITLE_MODE_NAME, MISC_EXPORT_TITLES_FIRST,
			MISC_EXPORT_TITLES_LAST, colTitle );
    rowGridPop	= new_popup( MISC_GRID_MODE_NAME, MISC_EXPORT_GRID_FIRST,
			MISC_EXPORT_GRID_LAST, rowGrid );
    colGridPop	= new_popup( MISC_GRID_MODE_NAME, MISC_EXPORT_GRID_FIRST,
			MISC_EXPORT_GRID_LAST, colGrid );

    formatBox	= new_box( formatPop,   "Format"        );
    rowTitleBox	= new_box( rowTitlePop, "Row Titles"    );
    colTitleBox	= new_box( colTitlePop, "Column Titles" );
    rowGridBox	= new_box( rowGridPop,  "Row Grid"      );
    colGridBox	= new_box( colGridPop,  "Column Grid"   );

    int const BOX_SPACING = 3;
    NXRect r; [formatBox getFrame:&r];		// Choose one box to represent
    NXCoord const boxHeight   = r.size.height;	// height of all boxes.
    NXCoord const titleWidth  = [self unifyWidths:rowTitleBox:colTitleBox];
    NXCoord const gridWidth   = [self unifyWidths:rowGridBox :colGridBox ];
    NXCoord const formatWidth = titleWidth + BOX_SPACING + gridWidth;

    NXCoord y = BOX_SPACING;
    [colTitleBox moveTo:0:y];
    [colGridBox  moveTo:titleWidth + BOX_SPACING:y];

    y += boxHeight + BOX_SPACING;
    [rowTitleBox moveTo:0:y];
    [rowGridBox  moveTo:titleWidth + BOX_SPACING:y];

    y += boxHeight + BOX_SPACING;
    [formatBox moveTo:0:y];
    [formatBox sizeTo:formatWidth:boxHeight];

    [self sizeTo:formatWidth:y + boxHeight];

    [self addSubview:formatBox];
    [self addSubview:rowTitleBox];
    [self addSubview:colTitleBox];
    [self addSubview:rowGridBox];
    [self addSubview:colGridBox];

    return self;
    }

@end
